@@ -24,20 +24,25 @@ import subprocess
 import tarfile
 import tempfile
 
-from debian_bundle.changelog import Version
+try:
+    from debian.changelog import Version
+except ImportError:
+    # Prior to 0.1.15 the debian module was called debian_bundle
+    from debian_bundle.changelog import Version
 
-from bzrlib.export import export
 from bzrlib.revisionspec import RevisionSpec
-from bzrlib.trace import info
+from bzrlib.trace import note
 
 from bzrlib.plugins.builddeb.errors import (
     MissingUpstreamTarball,
     PackageVersionNotPresent,
+    PerFileTimestampsNotSupported,
     PristineTarError,
     )
 from bzrlib.plugins.builddeb.import_dsc import DistributionBranch
 from bzrlib.plugins.builddeb.repack_tarball import repack_tarball
 from bzrlib.plugins.builddeb.util import (
+    export,
     get_snapshot_revision,
     tarball_name,
     )
@@ -46,13 +51,16 @@ from bzrlib.plugins.builddeb.util import (
 class UpstreamSource(object):
     """A source for upstream versions (uscan, get-orig-source, etc)."""
 
-    def get_latest_version(self, package, target_dir):
+    def get_latest_version(self, package, version, target_dir):
         """Fetch the source tarball for the latest available version.
 
         :param package: Name of the package
+        :param version: The current version of the package.
         :param target_dir: Directory in which to store the tarball
+        :return: The version number of the new version, or None if no newer
+                version is available.
         """
-        raise NotImplemented(self.get_latest_version)
+        raise NotImplementedError(self.get_latest_version)
 
     def get_specific_version(self, package, version, target_dir):
         """Fetch the source tarball for a particular version.
@@ -61,10 +69,11 @@ class UpstreamSource(object):
         :param version: Version string of the version to fetch
         :param target_dir: Directory in which to store the tarball
         """
-        raise NotImplemented(self.get_specific_version)
+        raise NotImplementedError(self.get_specific_version)
 
-    def _tarball_path(self, package, version, target_dir):
-        return os.path.join(target_dir, tarball_name(package, version))
+    def _tarball_path(self, package, version, target_dir, format=None):
+        return os.path.join(target_dir, tarball_name(package, version,
+                    format=format))
 
 
 class PristineTarSource(UpstreamSource):
@@ -75,19 +84,24 @@ class PristineTarSource(UpstreamSource):
         self.tree = tree
 
     def get_specific_version(self, package, version, target_dir):
-        target_filename = self._tarball_path(package, version,
-                                             target_dir)
         db = DistributionBranch(self.branch, None, tree=self.tree)
         if not db.has_upstream_version_in_packaging_branch(version):
             raise PackageVersionNotPresent(package, version, self)
         revid = db.revid_of_upstream_version_from_branch(version)
-        if not db.has_pristine_tar_delta(revid):
-            raise PackageVersionNotPresent(package, version, self)
-        info("Using pristine-tar to reconstruct the needed tarball.")
+        rev = self.branch.repository.get_revision(revid)
+        note("Using pristine-tar to reconstruct the needed tarball.")
+        if db.has_pristine_tar_delta(rev):
+            format = db.pristine_tar_format(rev)
+        else:
+            format = 'gz'
+        target_filename = self._tarball_path(package, version,
+                                             target_dir, format=format)
         try:
             db.reconstruct_pristine_tar(revid, package, version, target_filename)
         except PristineTarError:
             raise PackageVersionNotPresent(package, version, self)
+        except PerFileTimestampsNotSupported:
+            raise PackageVersionNotPresent(package, version, self)
 
 
 class AptSource(UpstreamSource):
@@ -101,22 +115,32 @@ class AptSource(UpstreamSource):
             apt_pkg = _apt_pkg
         apt_pkg.init()
 
+        def get_fn(obj, new_name, old_name):
+            try:
+                return getattr(obj, new_name)
+            except AttributeError:
+                return getattr(obj, old_name)
+
         # Handle the case where the apt.sources file contains no source
         # URIs (LP:375897)
         try:
-            sources = apt_pkg.GetPkgSrcRecords()
+            get_sources = get_fn(apt_pkg, 'SourceRecords',
+                "GetPkgSrcRecords")
+            sources = get_sources()
         except SystemError:
             raise PackageVersionNotPresent(package, upstream_version, self)
 
-        sources.Restart()
-        info("Using apt to look for the upstream tarball.")
-        while sources.Lookup(package):
-            if upstream_version \
-                == Version(sources.Version).upstream_version:
-                if self._run_apt_source(package, sources.Version, target_dir):
+        restart = get_fn(sources, 'restart', 'Restart')
+        restart()
+        note("Using apt to look for the upstream tarball.")
+        lookup = get_fn(sources, 'lookup', 'Lookup')
+        while lookup(package):
+            version = get_fn(sources, 'version', 'Version')
+            if upstream_version == Version(version).upstream_version:
+                if self._run_apt_source(package, version, target_dir):
                     return
                 break
-        info("apt could not find the needed tarball.")
+        note("apt could not find the needed tarball.")
         raise PackageVersionNotPresent(package, upstream_version, self)
 
     def _get_command(self, package, version_str):
@@ -133,31 +157,35 @@ class AptSource(UpstreamSource):
 
 
 class UpstreamBranchSource(UpstreamSource):
-    """Upstream source that uses the upstream branch."""
+    """Upstream source that uses the upstream branch.
+    
+    :ivar upstream_branch: Branch with upstream sources
+    :ivar upstream_version_map: Map from version strings to revids
+    """
 
-    def __init__(self, upstream_branch, upstream_revision=None, 
-                 fallback_revspec=None):
+    def __init__(self, upstream_branch, upstream_revision_map=None):
         self.upstream_branch = upstream_branch
-        self.upstream_revision = upstream_revision
-        self.fallback_revspec = fallback_revspec
+        if upstream_revision_map is None:
+            self.upstream_revision_map = {}
+        else:
+            self.upstream_revision_map = upstream_revision_map
 
     def _get_revision_id(self, version):
-        if self.upstream_revision is not None:
-            # Explicit revision id to use set
-            return self.upstream_revision
+        if version in self.upstream_revision_map:
+             return self.upstream_revision_map[version]
         revspec = get_snapshot_revision(version)
-        if revspec is None:
-            revspec = self.fallback_revspec
         if revspec is not None:
             return RevisionSpec.from_string(
                 revspec).as_revision_id(self.upstream_branch)
-        return self.upstream_branch.last_revision()
+        return None
 
     def get_specific_version(self, package, version, target_dir):
         self.upstream_branch.lock_read()
         try:
             revid = self._get_revision_id(version)
-            info("Exporting upstream branch revision %s to create the tarball",
+            if revid is None:
+                raise PackageVersionNotPresent(package, version, self)
+            note("Exporting upstream branch revision %s to create the tarball",
                  revid)
             target_filename = self._tarball_path(package, version, target_dir)
             tarball_base = "%s-%s" % (package, version)
@@ -174,22 +202,23 @@ class GetOrigSourceSource(UpstreamSource):
         self.tree = tree
         self.larstiq = larstiq
 
-    def _get_orig_source(self, source_dir, desired_tarball_name,
+    def _get_orig_source(self, source_dir, desired_tarball_names,
                         target_dir):
-        info("Trying to use get-orig-source to retrieve needed tarball.")
-        command = ["/usr/bin/make", "-f", "debian/rules", "get-orig-source"]
+        note("Trying to use get-orig-source to retrieve needed tarball.")
+        command = ["make", "-f", "debian/rules", "get-orig-source"]
         proc = subprocess.Popen(command, cwd=source_dir)
         ret = proc.wait()
         if ret != 0:
-            info("Trying to run get-orig-source rule failed")
+            note("Trying to run get-orig-source rule failed")
             return False
-        fetched_tarball = os.path.join(source_dir, desired_tarball_name)
-        if not os.path.exists(fetched_tarball):
-            info("get-orig-source did not create %s", desired_tarball_name)
-            return False
-        repack_tarball(fetched_tarball, desired_tarball_name, 
-                       target_dir=target_dir)
-        return True
+        for desired_tarball_name in desired_tarball_names:
+            fetched_tarball = os.path.join(source_dir, desired_tarball_name)
+            if os.path.exists(fetched_tarball):
+                repack_tarball(fetched_tarball, desired_tarball_name,
+                               target_dir=target_dir, force_gz=False)
+                return True
+        note("get-orig-source did not create %s", desired_tarball_name)
+        return False
 
     def get_specific_version(self, package, version, target_dir):
         if self.larstiq:
@@ -198,7 +227,9 @@ class GetOrigSourceSource(UpstreamSource):
             rules_name = 'debian/rules'
         rules_id = self.tree.path2id(rules_name)
         if rules_id is not None:
-            desired_tarball_name = tarball_name(package, version)
+            desired_tarball_names = [tarball_name(package, version),
+                    tarball_name(package, version, 'bz2'),
+                    tarball_name(package, version, 'lzma')]
             tmpdir = tempfile.mkdtemp(prefix="builddeb-get-orig-source-")
             try:
                 base_export_dir = os.path.join(tmpdir, "export")
@@ -208,12 +239,12 @@ class GetOrigSourceSource(UpstreamSource):
                     export_dir = os.path.join(export_dir, "debian")
                 export(self.tree, export_dir, format="dir")
                 if not self._get_orig_source(base_export_dir,
-                        desired_tarball_name, target_dir):
+                        desired_tarball_names, target_dir):
                     raise PackageVersionNotPresent(package, version, self)
                 return
             finally:
                 shutil.rmtree(tmpdir)
-        info("No debian/rules file to try and use for a get-orig-source rule")
+        note("No debian/rules file to try and use for a get-orig-source rule")
         raise PackageVersionNotPresent(package, version, self)
 
 
@@ -225,14 +256,14 @@ class UScanSource(UpstreamSource):
         self.larstiq = larstiq
 
     def _uscan(self, package, upstream_version, watch_file, target_dir):
-        info("Using uscan to look for the upstream tarball.")
+        note("Using uscan to look for the upstream tarball.")
         r = os.system("uscan --upstream-version %s --force-download --rename "
                       "--package %s --watchfile %s --check-dirname-level 0 " 
                       "--download --repack --destdir %s --download-version %s" %
                       (upstream_version, package, watch_file, target_dir,
                        upstream_version))
         if r != 0:
-            info("uscan could not find the needed tarball.")
+            note("uscan could not find the needed tarball.")
             return False
         return True
 
@@ -243,7 +274,7 @@ class UScanSource(UpstreamSource):
             watchfile = 'debian/watch'
         watch_id = self.tree.path2id(watchfile)
         if watch_id is None:
-            info("No watch file to use to retrieve upstream tarball.")
+            note("No watch file to use to retrieve upstream tarball.")
             return None
         (tmp, tempfilename) = tempfile.mkstemp()
         try:
@@ -259,11 +290,14 @@ class UScanSource(UpstreamSource):
         if tempfilename is None:
             raise PackageVersionNotPresent(package, version, self)
         try:
-            if not self._uscan(package, version, tempfilename, 
+            if not self._uscan(package, version, tempfilename,
                     target_dir):
                 raise PackageVersionNotPresent(package, version, self)
         finally:
-          os.unlink(tempfilename)
+            os.unlink(tempfilename)
+
+    def get_latest_version(self, package, version, target_dir):
+        pass
 
 
 class SelfSplitSource(UpstreamSource):
@@ -287,7 +321,7 @@ class SelfSplitSource(UpstreamSource):
             shutil.rmtree(tmpdir)
 
     def get_specific_version(self, package, version, target_dir):
-        info("Using the current branch without the 'debian' directory "
+        note("Using the current branch without the 'debian' directory "
                 "to create the tarball")
         self._split(package, version, 
                     self._tarball_path(package, version, target_dir))
@@ -313,36 +347,15 @@ class StackedUpstreamSource(UpstreamSource):
                 pass
         raise PackageVersionNotPresent(package, version, self)
 
-
-def get_upstream_sources(tree, branch, larstiq=False, upstream_branch=None,
-                         upstream_revision=None, allow_split=False):
-    """Get the list of upstream sources to retrieve upstream versions from.
-
-    :param tree: The tree that is being built from.
-    :param branch: The branch that is being built from.
-    :param larstiq: Whether the tree versions the root of ./debian.
-    :param upstream_branch: An upstream branch that can be exported
-        if needed.
-    :param upstream_revision: The revision to use of the upstream branch
-        if it is used.
-    :param allow_split: Whether the provider can provide the tarball
-        by exporting the branch and removing the "debian" dir.
-
-    """
-    sources = [
-        PristineTarSource(tree, branch), 
-        AptSource(),
-        ]
-    if upstream_branch is not None:
-        sources.append(
-            UpstreamBranchSource(upstream_branch, upstream_revision))
-    sources.extend([
-        GetOrigSourceSource(tree, larstiq), 
-        UScanSource(tree, larstiq),
-        ])
-    if allow_split:
-        sources.append(SelfSplitSource(tree))
-    return sources
+    def get_latest_version(self, package, version, target_dir):
+        for source in self._sources:
+            try:
+                new_version = source.get_latest_version(package, version, target_dir)
+                if new_version is not None:
+                    return new_version
+            except NotImplementedError:
+                pass
+        return None
 
 
 class UpstreamProvider(object):
@@ -387,41 +400,53 @@ class UpstreamProvider(object):
         :param target_dir: The directory to place the tarball in.
         :return: The path to the tarball.
         """
-        info("Looking for a way to retrieve the upstream tarball")
-        if self.already_exists_in_target(target_dir):
-            info("Upstream tarball already exists in build directory, "
+        note("Looking for a way to retrieve the upstream tarball")
+        in_target = self.already_exists_in_target(target_dir)
+        if in_target is not None:
+            note("Upstream tarball already exists in build directory, "
                     "using that")
-            return os.path.join(target_dir, self._tarball_name())
-        if not self.already_exists_in_store():
+            return in_target
+        if self.already_exists_in_store() is None:
             if not os.path.exists(self.store_dir):
                 os.makedirs(self.store_dir)
             try:
-                self.source.get_specific_version(self.package, 
+                self.source.get_specific_version(self.package,
                     self.version.upstream_version, self.store_dir)
             except PackageVersionNotPresent:
-                raise MissingUpstreamTarball(self._tarball_name())
+                raise MissingUpstreamTarball(self._tarball_names()[0])
         else:
-             info("Using the upstream tarball that is present in "
+             note("Using the upstream tarball that is present in "
                      "%s" % self.store_dir)
-        assert self.provide_from_store_dir(target_dir)
-        return os.path.join(target_dir, self._tarball_name())
+        path = self.provide_from_store_dir(target_dir)
+        assert path is not None
+        return path
 
     def already_exists_in_target(self, target_dir):
-        return os.path.exists(os.path.join(target_dir, self._tarball_name()))
+        for tarball_name in self._tarball_names():
+            path = os.path.join(target_dir, tarball_name)
+            if os.path.exists(path):
+                return path
+        return None
 
     def already_exists_in_store(self):
-        return os.path.exists(os.path.join(self.store_dir,
-                    self._tarball_name()))
+        for tarball_name in self._tarball_names():
+            path = os.path.join(self.store_dir, tarball_name)
+            if os.path.exists(path):
+                return path
+        return None
 
     def provide_from_store_dir(self, target_dir):
-        if self.already_exists_in_store():
-            repack_tarball(os.path.join(self.store_dir, self._tarball_name()),
-                    self._tarball_name(), target_dir=target_dir)
-            return True
-        return False
+        path = self.already_exists_in_store()
+        if path is not None:
+            repack_tarball(path, os.path.basename(path),
+                    target_dir=target_dir, force_gz=False)
+            return path
+        return path
 
-    def _tarball_name(self):
-        return tarball_name(self.package, self.version.upstream_version)
+    def _tarball_names(self):
+        return [tarball_name(self.package, self.version.upstream_version),
+                tarball_name(self.package, self.version.upstream_version, format='bz2'),
+                tarball_name(self.package, self.version.upstream_version, format='lzma')]
 
 
 class _MissingUpstreamProvider(UpstreamProvider):
@@ -455,7 +480,8 @@ class _SimpleUpstreamProvider(UpstreamProvider):
         self.store_dir = store_dir
 
     def provide(self, target_dir):
-        if self.already_exists_in_target(target_dir) \
-            or self.provide_from_store_dir(target_dir):
-            return os.path.join(target_dir, self._tarball_name())
-        raise MissingUpstreamTarball(self._tarball_name())
+        path = (self.already_exists_in_target(target_dir)
+                or self.provide_from_store_dir(target_dir))
+        if path is not None:
+            return path
+        raise MissingUpstreamTarball(self._tarball_names()[0])
