@@ -22,6 +22,7 @@ try:
     import hashlib as md5
 except ImportError:
     import md5
+import signal
 import shutil
 import tempfile
 import os
@@ -29,25 +30,69 @@ import re
 
 from bzrlib.trace import mutter
 
-from debian_bundle import deb822
-from debian_bundle.changelog import Changelog, ChangelogParseError
+try:
+    from debian import deb822
+    from debian.changelog import Changelog, ChangelogParseError
+except ImportError:
+    # Prior to 0.1.15 the debian module was called debian_bundle
+    from debian_bundle import deb822
+    from debian_bundle.changelog import Changelog, ChangelogParseError
 
 from bzrlib import (
         bugtracker,
         errors,
         urlutils,
+        version_info as bzr_version_info,
         )
+from bzrlib.export import export as bzr_export
+from bzrlib.trace import warning
 from bzrlib.transport import (
     do_catching_redirections,
     get_transport,
     )
+from bzrlib.plugins.builddeb import (
+    default_conf,
+    local_conf,
+    global_conf,
+    )
+from bzrlib.plugins.builddeb.config import DebBuildConfig
 from bzrlib.plugins.builddeb.errors import (
                 MissingChangelogError,
                 AddChangelogError,
+                NoPreviousUpload,
+                UnableToFindPreviousUpload,
+                UnknownDistribution,
                 UnparseableChangelog,
                 )
 
 
+DEBIAN_RELEASES = ('woody', 'sarge', 'etch', 'lenny', 'squeeze', 'stable',
+        'testing', 'unstable', 'experimental', 'frozen', 'sid')
+DEBIAN_POCKETS = ('', '-security', '-proposed-updates', '-backports')
+UBUNTU_RELEASES = ('warty', 'hoary', 'breezy', 'dapper', 'edgy',
+        'feisty', 'gutsy', 'hardy', 'intrepid', 'jaunty', 'karmic',
+        'lucid', 'maverick', 'natty')
+UBUNTU_POCKETS = ('', '-proposed', '-updates', '-security', '-backports')
+
+
+def safe_decode(s):
+    """Decode a string into a Unicode value."""
+    if isinstance(s, unicode): # Already unicode
+        mutter('safe_decode() called on an already-unicode string: %r' % (s,))
+        return s
+    try:
+        return s.decode('utf-8')
+    except UnicodeDecodeError, e:
+        mutter('safe_decode(%r) falling back to iso-8859-1' % (s,))
+        # TODO: Looking at BeautifulSoup it seems to use 'chardet' to try to
+        #       guess the encoding of a given text stream. We might want to
+        #       take a closer look at that.
+        # TODO: Another possibility would be to make the fallback encoding
+        #       configurable, possibly exposed as a command-line flag, for now,
+        #       this seems 'good enough'.
+        return s.decode('iso-8859-1')
+
+
 def recursive_copy(fromdir, todir):
     """Copy the contents of fromdir to todir.
 
@@ -83,7 +128,8 @@ def find_changelog(t, merge, max_blocks=1):
 
     :param t: the Tree to look in.
     :param merge: whether this is a "merge" package.
-    :param max_blocks: Number of max_blocks to parse (defaults to 1)
+    :param max_blocks: Number of max_blocks to parse (defaults to 1). Use None
+        to parse the entire changelog.
     :return: (changelog, larstiq) where changelog is the Changelog,
         and larstiq is a boolean indicating whether the file is at
         'changelog' if merge was given, False otherwise.
@@ -154,14 +200,18 @@ def strip_changelog_message(changes):
         return changes
 
 
-def tarball_name(package, version):
+def tarball_name(package, version, format=None):
     """Return the name of the .orig.tar.gz for the given package and version.
 
     :param package: the name of the source package.
     :param version: the upstream version of the package.
+    :param format: the format for the tarball. If None then 'gz' will be
+         used. You probably want on of 'gz', 'bz2', or 'lzma'.
     :return: a string that is the name of the upstream tarball to use.
     """
-    return "%s_%s.orig.tar.gz" % (package, str(version))
+    if format is None:
+        format = 'gz'
+    return "%s_%s.orig.tar.%s" % (package, str(version), format)
 
 
 def get_snapshot_revision(upstream_version):
@@ -209,15 +259,8 @@ def suite_to_distribution(suite):
     :param suite: the string containing the suite
     :return: "debian", "ubuntu", or None if the distribution couldn't be inferred.
     """
-    debian_releases = ('woody', 'sarge', 'etch', 'lenny', 'squeeze', 'stable',
-            'testing', 'unstable', 'experimental', 'frozen')
-    debian_targets = ('', '-security', '-proposed-updates', '-backports')
-    ubuntu_releases = ('warty', 'hoary', 'breezy', 'dapper', 'edgy',
-            'feisty', 'gutsy', 'hardy', 'intrepid', 'jaunty', 'karmic',
-            'lucid')
-    ubuntu_targets = ('', '-proposed', '-updates', '-security', '-backports')
-    all_debian = [r + t for r in debian_releases for t in debian_targets]
-    all_ubuntu = [r + t for r in ubuntu_releases for t in ubuntu_targets]
+    all_debian = [r + t for r in DEBIAN_RELEASES for t in DEBIAN_POCKETS]
+    all_ubuntu = [r + t for r in UBUNTU_RELEASES for t in UBUNTU_POCKETS]
     if suite in all_debian:
         return "debian"
     if suite in all_ubuntu:
@@ -237,18 +280,24 @@ def lookup_distribution(distribution_or_suite):
     return suite_to_distribution(distribution_or_suite)
 
 
+def md5sum_filename(filename):
+    """Calculate the md5sum of a file by name."""
+    m = md5.md5()
+    f = open(filename, 'rb')
+    try:
+        for line in f:
+            m.update(line)
+    finally:
+        f.close()
+    return m.hexdigest()
+
+
 def move_file_if_different(source, target, md5sum):
     if os.path.exists(target):
         if os.path.samefile(source, target):
             return
-        t_md5sum = md5.md5()
-        target_f = open(target)
-        try:
-            for line in target_f:
-                t_md5sum.update(line)
-        finally:
-            target_f.close()
-        if t_md5sum.hexdigest() == md5sum:
+        t_md5sum = md5sum_filename(target)
+        if t_md5sum == md5sum:
             return
     shutil.move(source, target)
 
@@ -380,36 +429,36 @@ def find_bugs_fixed(changes, branch, _lplib=None):
 
 
 def find_extra_authors(changes):
-    extra_author_re = re.compile(r"\s*\[([^\]]+)]\s*", re.UNICODE)
+    extra_author_re = re.compile(r"\s*\[([^\]]+)]\s*")
     authors = []
     for change in changes:
         # Parse out any extra authors.
-        match = extra_author_re.match(change.decode("utf-8"))
+        match = extra_author_re.match(change)
         if match is not None:
-            new_author = match.group(1).strip()
+            new_author = safe_decode(match.group(1).strip())
             already_included = False
             for author in authors:
                 if author.startswith(new_author):
                     already_included = True
                     break
             if not already_included:
-                authors.append(new_author.encode("utf-8"))
+                authors.append(new_author)
     return authors
 
 
 def find_thanks(changes):
     thanks_re = re.compile(r"[tT]hank(?:(?:s)|(?:you))(?:\s*to)?"
-            "((?:\s+(?:(?:[A-Z]\.)|(?:[A-Z]\w+(?:-[A-Z]\w+)*)))+"
+            "((?:\s+(?:(?:\w\.)|(?:\w+(?:-\w+)*)))+"
             "(?:\s+<[^@>]+@[^@>]+>)?)",
             re.UNICODE)
     thanks = []
-    changes_str = " ".join(changes).decode("utf-8")
+    changes_str = safe_decode(" ".join(changes))
     for match in thanks_re.finditer(changes_str):
         if thanks is None:
             thanks = []
         thanks_str = match.group(1).strip()
         thanks_str = re.sub(r"\s+", " ", thanks_str)
-        thanks.append(thanks_str.encode("utf-8"))
+        thanks.append(thanks_str)
     return thanks
 
 
@@ -434,12 +483,12 @@ def get_commit_info_from_changelog(changelog, branch, _lplib=None):
     bugs = []
     if changelog._blocks:
         block = changelog._blocks[0]
-        authors = [block.author]
+        authors = [safe_decode(block.author)]
         changes = strip_changelog_message(block.changes())
         authors += find_extra_authors(changes)
         bugs = find_bugs_fixed(changes, branch, _lplib=_lplib)
         thanks = find_thanks(changes)
-        message = "\n".join(changes)
+        message = safe_decode("\n".join(changes).replace("\r", ""))
     return (message, authors, thanks, bugs)
 
 
@@ -447,7 +496,7 @@ def find_last_distribution(changelog):
     """Find the last changelog that was used in a changelog.
 
     This will skip stanzas with the 'UNRELEASED' distribution.
-    
+
     :param changelog: Changelog to analyze
     """
     for block in changelog._blocks:
@@ -455,3 +504,108 @@ def find_last_distribution(changelog):
         if distribution != "UNRELEASED":
             return distribution
     return None
+
+
+def subprocess_setup():
+    # Python installs a SIGPIPE handler by default. This is usually not what
+    # non-Python subprocesses expect.
+    # Many, many thanks to Colin Watson
+    signal.signal(signal.SIGPIPE, signal.SIG_DFL)
+
+
+def debuild_config(tree, working_tree):
+    """Obtain the Debuild configuration object.
+
+    :param tree: A Tree object, can be a WorkingTree or RevisionTree.
+    :param working_tree: Whether the tree is a working tree.
+    """
+    config_files = []
+    user_config = None
+    if (working_tree and tree.has_filename(local_conf)):
+        if tree.path2id(local_conf) is None:
+            config_files.append((tree.get_file_byname(local_conf), True,
+                        "local.conf"))
+        else:
+            warning('Not using configuration from %s as it is versioned.')
+    config_files.append((global_conf(), True))
+    user_config = global_conf()
+    if tree.path2id(default_conf):
+        config_files.append((tree.get_file(tree.path2id(default_conf)), False,
+                    "default.conf"))
+    config = DebBuildConfig(config_files, tree=tree)
+    config.set_user_config(user_config)
+    return config
+
+
+def export(tree, dest, format=None, root=None, subdir=None, filtered=False,
+           require_per_file_timestamps=False):
+    """Simple wrapper around bzrlib.export.export that prefers 
+    per_file_timestamps if it is supported.
+
+    """
+    # per_file_timestamps is available as of bzr 2.2.0
+    if bzr_version_info >= (2, 2, 0):
+        return bzr_export(tree, dest, format=format, root=root, subdir=subdir,
+            filtered=filtered, per_file_timestamps=True)
+    else:
+        if require_per_file_timestamps:
+            raise errors.PerFileTimestampsNotSupported()
+        return bzr_export(tree, dest, format=format, root=root, subdir=subdir,
+            filtered=filtered)
+
+
+def find_previous_upload(tree, merge):
+    """Given a tree, find the previous upload to the distribution.
+
+    When e.g. Ubuntu merges from Debian they want to build with
+    -vPREV_VERSION. Here's where we find that previous version.
+
+    We look at the last changelog entry and find the upload target.
+    We then search backwards until we find the same target. That's
+    the previous version that we return.
+
+    We require there to be a previous version, otherwise we throw
+    an error.
+
+    It's not a simple string comparison to find the same target in
+    a previous version, as we should consider old series in e.g.
+    Ubuntu.
+    """
+    try:
+        cl, larstiq = find_changelog(tree, merge, max_blocks=None)
+    except UnparseableChangelog:
+        raise UnableToFindPreviousUpload()
+    return _find_previous_upload(cl)
+
+
+def _find_previous_upload(cl):
+    blocks = cl._blocks
+    current_target = blocks[0].distributions.split(" ")[0]
+    all_debian = [r + t for r in DEBIAN_RELEASES for t in DEBIAN_POCKETS]
+    all_ubuntu = [r + t for r in UBUNTU_RELEASES for t in UBUNTU_POCKETS]
+    if current_target in all_debian:
+        match_targets = (current_target,)
+    elif current_target in all_ubuntu:
+        match_targets = UBUNTU_RELEASES
+        if "-" in current_target:
+            match_targets += [current_target.split("-", 1)[0]
+                + t for t in UBUNTU_POCKETS]
+    else:
+        raise UnknownDistribution(current_target)
+    previous_version = None
+    for block in blocks[1:]:
+        if block.distributions.split(" ")[0] in match_targets:
+            return block.version
+    raise NoPreviousUpload(current_target)
+
+
+def tree_contains_upstream_source(tree):
+    """Guess if the specified tree contains the upstream source.
+    
+    :param tree: A RevisionTree.
+    :return: Boolean indicating whether or not the tree contains the upstream
+        source
+    """
+    present_files = set(tree.inventory.root.children.keys())
+    packaging_files = frozenset(["debian", ".bzr-builddeb"])
+    return (len(present_files - packaging_files) > 0)
